import os
import sys

SakRootPath = ""


def getPath(config_name=""):
    """获取路径 返回运行本函数的文件绝对路径+config_name
    determine if application is a script file or frozen exe
    """
    if getattr(sys, 'frozen', False):
        application_path = os.path.dirname(sys.executable)
    elif __file__:
        application_path = os.path.dirname(__file__)
    config_path = os.path.join(application_path, config_name)
    return config_path


# 设置项目的根目录
def setRootPath(path: str):
    """
        设置项目的根目录
    @param path:
    @return:
    """
    global SakRootPath
    SakRootPath = path
    return True


# 获取项目根目录 PS：调用前需要先setRootPath
def getRootPath():
    """
        获取项目根目录
    :return:
    """
    global SakRootPath
    if SakRootPath == "":
        print("请先调用 setRootPath 设置项目根目录")
        return False
    return SakRootPath


# 检测目录/文件是否存在
def isPathExists(path=""):
    """
        检测 目录 或 文件 是否存在
    :param path: 路径
    :return:
    """
    if path == "":
        return False

    return os.path.exists(path)


# 创建目录 PS：若路径不存在，则递归创建，若存在则不执行）
def mkdir(path: str):
    """
        递归创建目录
        1、若路径不存在，则递归创建。
        2、若路径已存在，则不执行。
    :param path: 路径
    :return:
    """
    if not isPathExists(path):
        os.makedirs(path)
        return True
    else:
        return False



# 日志等级常量
INFO_LOG_LEVEL = "INFO"
WARNING_LOG_LEVEL = "WARNING"
ERROR_LOG_LEVEL = "ERROR"


# 输出日志 INFO级显示为绿色、WARNING级显示为黄色、ERROR级显示为红色
def showLog(msg="", title="", level=INFO_LOG_LEVEL, showEmptyLine=False):
    """
        # print("\033[1;32m 字体颜色：绿色\033[0m")
        # print("\033[1;33m 字体颜色：浅黄色\033[0m")
        # print("\033[1;31m 字体颜色：红色\033[0m")

    :param title: 标题
    :param msg: 信息
    :param level: 日志等级
    :param showEmptyLine: 是否显示空行
    :return:
    """
    # 根据日志等级更换颜色显示
    if level == INFO_LOG_LEVEL:
        prefix = "\033[1;32m"
    elif level == WARNING_LOG_LEVEL:
        prefix = "\033[1;33m"
    elif level == ERROR_LOG_LEVEL:
        prefix = "\033[1;31m"

    suffix = "\033[0m"

    if title != "":
        print(f"{prefix}[ {level} ]: {title}{suffix}")
    else:
        print(f"{prefix}[ {level} ]: {msg}{suffix}")
        return True

    if msg != "":
        print(f"{prefix}{msg}{suffix}")

    if showEmptyLine:
        print()


# 关闭 InsecureRequestWarning
def hideInsecureRequestWarning():
    """
    关闭 InsecureRequestWarning
    :return:
    """
    import urllib3
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)



import configparser
from pysak.path import *


def getConfig(configPath="config\\main.ini", section=""):
    """
        读取指定的ini配置文件

    :param configPath: 配置文件相对路径 如：config\\main.ini
    :param section: 配置文件下的section
    :return:
    """
    config = configparser.ConfigParser()
    config.read_file(open(f"{getRootPath()}{configPath}", encoding='UTF-8'))

    # 获取conf中的section
    sections = config.sections()

    configs = {}

    for index in range(len(sections)):
        configs[sections[index]] = {}

    for index in range(len(configs)):
        sectionName = sections[index]

        # 当前section的所有option
        options = config.options(sectionName)
        for index in range(len(options)):
            # configs[sectionName][options[index]] = {}
            configs[sectionName][options[index]] = str(config.get(sectionName, options[index]))

    # 获取version的值 方法1
    # version = str(config.get(sections[0], paramList[0]))
    # 获取version的值 方法2
    # version = str(config.get("EditionInfo", "version"))
    return configs


def editConfig(section="", option="", value="", configPath="config\\main.ini"):
    """
        修改section下option的值为value
    :param section: 配置文件section
    :param option: section下的option
    :param value: 需要修改成什么值
    :param configPath: 配置文件相对路径 如：config\\main.ini
    :return:
    """
    config = configparser.ConfigParser()
    config.read_file(open(f"{getRootPath()}{configPath}", encoding='UTF-8'))

    # 获取conf中的section
    sections = config.sections()

    # 修改section下option的值为value
    config.set(section, option, str(value))
    config.write(open(configPath, "w"))



from .path import *

import json


# dict存入本地json文件
def dict2jsonFile(data={"a": 1, "b": 2, "c": 3}, fileName="data.json", RelativePath="\\"):
    """
    dict转json文件
    @param data: dict字典类型数据
    @param fileName: 生成的文件名
    @param RelativePath: 生成文件的相对路径
    @return:
    """
    # 获取项目根目录
    ROOT_PATH = getRootPath()

    if ROOT_PATH == False:
        return False

    if RelativePath != "":
        PATH = f"{ROOT_PATH}{RelativePath}"
    else:
        PATH = ROOT_PATH
    mkdir(PATH)

    json_str = json.dumps(data, indent=4)

    with open(f"{PATH}\\{fileName}", 'w') as json_file:
        json_file.write(json_str)
        # json.dump(json_str, json_file)

    return True


# json文件内容转dict
def jsonFile2dict(fileName="data.json", RelativePath="\\"):
    """

    @param fileName:
    @param RelativePath:
    @return:
    """
    # 获取项目根目录
    ROOT_PATH = getRootPath()

    if ROOT_PATH == False:
        return False

    if RelativePath != "":
        PATH = f"{ROOT_PATH}{RelativePath}"
    else:
        PATH = ROOT_PATH

    filePath = f"{PATH}\\{fileName}"

    # 判断文件是否存在
    if not isPathExists(filePath):
        print(f"文件不存在：{filePath}")
        return False

    # 读取json文件，并转换为dict
    with open(filePath, 'r') as json_file:
        new_dict = json.load(json_file)

    # 字符串转dict
    # new_dict = json.loads(json_str)

    return new_dict


# 将字符串存入本地文件
def str2file(content="hello world!", fileName="data.txt", RelativePath="\\"):
    # 获取项目根目录
    ROOT_PATH = getRootPath()

    if ROOT_PATH == False:
        return False

    if RelativePath != "":
        PATH = f"{ROOT_PATH}{RelativePath}"
    else:
        PATH = ROOT_PATH
    mkdir(PATH)

    with open(f"{PATH}\\{fileName}", 'w') as file:
        file.write(content)

    return True



r"""
pysak is a simple, powerful and clean python common function library.

Github Repository: https://github.com/1Gbps/pysak
"""

SakUserPlatform = None


# 检查 python 版本
def checkPythonVersion():
    import platform

    pythonVersion = platform.python_version()
    pythonVersion = pythonVersion.split(".")

    MajorVersionNumber = int(pythonVersion[0])  # 主版本号
    MinorVersionNumber = int(pythonVersion[1])  # 子版本号
    RevisionNumber = int(pythonVersion[2])  # 修正版本号

    if MajorVersionNumber < 3:
        print("Sak 暂不兼容 python 3 以下的版本")
        exit()


# 获取 Platform 信息
def getUserPlatform():
    import platform

    global SakUserPlatform
    SakUserPlatform = platform.uname()

    return True


checkPythonVersion()
getUserPlatform()

from .basic import *
from .config import *
from .debug import *
from .file import *
from .img import *
from .MyThread import *
from .path import *
from .time import *
from .user_agent import *



import pysak
import time


# 获取当前时刻指定长度的时间戳  10位~17位
def getTimestamp(length=10):
    """
        获取当前时刻指定长度的时间戳  10位~17位
    :param length:
    :return:
    """

    if length > 17 or length < 10:
        return False

    # 计算差值
    cha = length - 10
    timestamp = pysak.time()

    if cha == 0:
        timestamp = int(timestamp)
    elif cha > 0:
        timestamp = int(timestamp * pow(10, cha))

    return timestamp


# 获取当前Date
def getDate(format="%Y-%m-%d %H:%M:%S"):
    """
    获取当前Date
    @param format: 日期格式
    @return:
    """
    ltime = time.localtime(getTimestamp(10))
    return time.strftime(format, ltime)



import requests


# 保存远程URL图片到本地
def saveImg(url, fileName="file", path=".\\"):
    """
        保存图片到本地
    :param url:
    :param fileName:
    :param path:
    :return:
    """
    response = requests.get(url, stream=True, verify=False)
    with open(f"{path}\\{fileName}", 'wb') as file:
        for chunk in response.iter_content():
            file.write(chunk)
    return True



import random


# 获取随机UserAgent
def getRandomUserAgent(browser="random"):
    """
    获取随机UserAgent PS：支持指定浏览器
    @param browser: 默认random，可选项：chrome、firefox、opera、ie
    @return:
    """
    UserAgent = [
        # chrome
        [
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1207.1 Safari/537.1",
            "Mozilla/5.0 (X11; CrOS i686 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1092.0 Safari/536.6",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1090.0 Safari/536.6",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5",
            "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.5",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.0 Safari/536.3",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
        ],

        # firefox
        [
            "Mozilla/5.0 (Macintosh; U; Mac OS X Mach-O; en-US; rv:2.0a) Gecko/20040614 Firefox/3.0.0 ",
            "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.0.3) Gecko/2008092414 Firefox/3.0.3",
            "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1) Gecko/20090624 Firefox/3.5",
            "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.14) Gecko/20110218 AlexaToolbar/alxf-2.0 Firefox/3.6.14",
            "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.2.15) Gecko/20110303 Firefox/3.6.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/20100101 Firefox/4.0.1",
        ],

        # Opera
        [
            "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10",
            "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8",
            "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        ],

        # ie
        [
            "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0",
            "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)",
            "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)",
            "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)",
        ]
    ]

    if browser == "random":
        # 随机选择浏览器
        browserIndex = random.randint(0, len(UserAgent) - 1)
    elif browser == "chrome":
        browserIndex = 0
    elif browser == "firefox":
        browserIndex = 1
    elif browser == "opera":
        browserIndex = 2
    elif browser == "ie":
        browserIndex = 3
    else:
        # 随机选择浏览器
        browserIndex = random.randint(0, len(UserAgent) - 1)

    # 随机选择所属浏览器的UserAgent
    UserAgentIndex = random.randint(0, len(UserAgent[browserIndex]) - 1)

    return UserAgent[browserIndex][UserAgentIndex]



import threading


class MyThread(threading.Thread):
    def __init__(self, name):
        """
        :param name: 线程名
        """
        threading.Thread.__init__(self)
        self.name = name

    def run(self):
        print(f"线程 {self.name} 运行中")



import json


# json转dict
def json2dict(jsonStr="{\"a\": 1, \"b\": 2, \"c\": 3}"):
    """
    json转dict
    @param jsonStr:
    @return:
    """
    return json.loads(jsonStr)


# dict转json
def dict2json(dict={"a": 1, "b": 2, "c": 3}):
    """
    dict转json
    @param dict:
    @return:
    """
    return json.dumps(dict)
